# multi-model data loading
setwd("/gpfs/scratch/bsc32/MN4/bsc32/bsc32801/amd_gitlab/sunset")
source("modules/Loading/Loading.R")
source("modules/Anomalies/Anomalies.R")

models <- c("ECMWF-SEAS5.1", "DWD-GCFS2.1")
vars <- c("tas")  # 'tas', 'tasmax', 'tasmin', 'prlr','sfcWind'
freq <- "monthly_mean" ## daily, daily_mean or weekly_mean
REF <- "CERRA"
months <- 1:12
lead <- 0:3
latmin = 39 # large-scale variable. For MOS no needed.
latmax = 44 # large-scale variable. For MOS no needed.
lonmin = -1 # large-scale variable. For MOS no needed.
lonmax = 4 # large-scale variable. For MOS no needed.
hcst_start <- '1993'
hcst_end <- '2016'
anomaly <- TRUE
anomaly_cross <- TRUE

#outpath <- "/gpfs/scratch/bsc32/MN4/bsc32/bsc32801/amd_gitlab/amd_ws/aspect/data/"

# to prevent typos
vars <- tolower(vars)
vars [which(vars == "sfcwind")] <- "sfcWind"

recipe_file <- paste0("/gpfs/scratch/bsc32/MN4/bsc32/bsc32801/amd_gitlab/amd_ws/aspect/",
                        "recipe_codorniu_downscaling_seasonal.yml")
recipe <- prepare_outputs(recipe_file)
recipe$Analysis$Variables$freq <- freq
recipe$Analysis$Time$hcst_start <- hcst_start
recipe$Analysis$Time$hcst_end <- hcst_end
recipe$Analysis$Region$latmin <- latmin
recipe$Analysis$Region$latmax <- latmax
recipe$Analysis$Region$lonmin <- lonmin
recipe$Analysis$Region$lonmax <- lonmax

for (var in vars) {

  recipe$Analysis$Variables$name <- var
  recipe$Analysis$Datasets$Reference$name <- REF

  if (var=="prlr" & toupper(REF) == "CERRA") {
    recipe$Analysis$Datasets$Reference$name <- "CERRA-Land"
  }

  for (target_month in months) {

    for (lt in lead) {

      diffmon <- target_month - lt
      smonth <- paste0(sprintf('%02d', ifelse(diffmon > 0, diffmon, 12 + diffmon)), '01')
      recipe$Analysis$Time$sdate <- smonth
      recipe$Analysis$Time$ftime_min <- lt+1
      recipe$Analysis$Time$ftime_max <- lt+1

      for (model in models) {

        recipe$Analysis$Datasets$System <- model
        data <- Loading(recipe)

	if (anomaly) {
          if (anomaly_cross) {
  	    year_num <- dim(data$hcst$data)["syear"]
            dum <- data
            for (i in 1:year_num) {
              tr_hcst <- Subset(data$hcst$data, along= "syear", indices=(1:year_num)[-i], drop=FALSE)
              te_hcst <- Subset(data$hcst$data, along= "syear", indices=i, drop=FALSE)
              tr_obs <- Subset(data$obs$data, along= "syear", indices=(1:year_num)[-i], drop=FALSE)
              te_obs <- Subset(data$obs$data, along= "syear", indices=i, drop=FALSE)

	      clim_tr_obs <- MeanDims(tr_obs, c('syear', "ensemble"))
              clim_tr_hcst <- MeanDims(tr_hcst, c('syear', 'ensemble'))

              ano_te_hcst <- s2dv::Ano(te_hcst, clim_tr_hcst,
                                       ncores = 4)
              ano_te_obs <- s2dv::Ano(te_obs, clim_tr_obs,
                                      ncores = 4)

              syear_dim <- which(names(dim(dum$hcst$data)) == "syear")
              index <- rep(list(quote(expr = )), length(dim(dum$hcst$data)))
	      index[syear_dim] <- i
              dum$hcst$data <- do.call(`[<-`, c(list(dum$hcst$data), index, list(ano_te_hcst)))
            
              syear_dim <- which(names(dim(dum$obs$data)) == "syear")
              index <- rep(list(quote(expr = )), length(dim(dum$obs$data)))
              index[syear_dim] <- i
              dum$obs$data <- do.call(`[<-`, c(list(dum$obs$data), index, list(ano_te_obs)))
	    }
  	    data <- dum
	    rm(dum)
  	  } else {
            data <- Anomalies (recipe, data)
          }
        }
      }
    }
  }
}



